/* logjam - a GTK client for LiveJournal.
 * Copyright (C) 2000-2003 Evan Martin <evan@livejournal.com>
 *
 * vim: tabstop=4 shiftwidth=4 noexpandtab :
 */

#include <glib.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

#include "util.h"
#include "conf_xml.h"
#include "conf.h"
#include "livejournal.h"

Configuration conf;
Application app;

#define PATH_BUF_SIZE 1024

Server*
conf_server_by_name(Configuration *c, const char *servername) {
	GList *l;
	for (l = c->servers; l != NULL; l = l->next) {
		if (strcmp(servername, ((Server*)l->data)->name) == 0) {
			return l->data;
		}
	}
	return NULL;
}

gboolean
conf_verify_dir(void) {
	return verify_dir(app.conf_dir, NULL);
}

void
conf_verify_a_server_exists() {
	if (conf.servers == NULL) {
		/* make a default server. */
		Server *s = server_new();
		s->name = g_strdup("LiveJournal.com");
		s->url = g_strdup("http://www.livejournal.com");
		conf.servers = g_list_append(conf.servers, s);
	}
}

void
conf_make_path(char *file, char *buf, int len) {
	char *path;
	path = g_build_filename(app.conf_dir, file, NULL);
	strncpy(buf, path, len);
	g_free(path);
}

char*
conf_make_curuser_path(const char *path) {
	return g_build_filename(app.conf_dir,
			"servers", conf_cur_server()->name,
			"users", conf_cur_user()->username,
			path ? path : NULL,
			NULL);
}

gboolean
conf_rename_server(Server *server, const char *newname, GError **err) {
	char *oldpath, *newpath;

	/* disallow:
	 *   [empty string]
	 *   .
	 *   ..
	 *   ./../foo
	 *   /foo
	 * allow:
	 *   .lAmE.sErVeR.
	 */
	if ((newname[0] == 0) ||
			(newname[0] == '.' &&
				(newname[1] == '.' || newname[1] == '/' || newname[1] == 0)) ||
			(newname[0] == '/')) {
		g_set_error(err, 0, 0, _("new server name is invalid"));
		return FALSE;
	}
	oldpath = g_build_filename(app.conf_dir, "servers", server->name, NULL);
	if (!g_file_test(oldpath, G_FILE_TEST_EXISTS)) {
		string_replace(&server->name, g_strdup(newname));
		g_free(oldpath);
		return TRUE;
	}
	newpath = g_build_filename(app.conf_dir, "servers", newname, NULL);
	if (rename(oldpath, newpath) < 0) {
		/* XXX this won't i18n properly in languages where
		 * the order of oldpath/newpath is different. */
		g_set_error(err, G_FILE_ERROR, g_file_error_from_errno(errno),
				_("renaming '%s' to '%s': %s"), oldpath, newpath,
				g_strerror(errno));
		g_free(oldpath);
		g_free(newpath);
		return FALSE;
	}
	string_replace(&server->name, g_strdup(newname));
	g_free(oldpath);
	g_free(newpath);
	return TRUE;
}

