/* logjam - a GTK client for LiveJournal.
 * Copyright (C) 2000-2003 Evan Martin <evan@livejournal.com>
 *
 * vim: tabstop=4 shiftwidth=4 noexpandtab :
 */

#ifndef lj_h
#define lj_h

#include "logjam-doc.h"
#include "livejournal.h"
#include "checkfriends.h"
#include "undo-manager.h"

typedef enum {
	TEXTSTYLE_HTML,
	TEXTSTYLE_STRUCTURED
} TextStyle;

typedef struct {
	GtkWindow win; /* super class */

	LogJamDoc *doc;

	GtkItemFactory *factory;

	TextStyle textstyle;

	/* menu items. */
	GtkWidget *mweb, *mloginout, *msubmitsep, *msubmit, *msaveserver; 
	GtkWidget *musejournal, *musejournalsep, *musejournal_defaultjournal;
	GtkWidget *mpicture;
	GtkWidget *mundo, *mredo;

	GtkWidget *actionbox;
	GtkWidget *usejournal; /* shows current usejournal */
	GtkWidget *baction, *laction; /* "action" button and its label: 
	                                 submit / save changes */
	GtkWidget *bdelete; /* "delete" button */
	GtkWidget *eentry;

	GtkWidget *metamgr;
	CFMgr* cfmgr;
	CFIndicator *cfi_main, *cfi_float;

	gpointer preview; /* we only want one preview window per ljwin. */
	
	UndoManager *um;
} LJWin;

void lj_font_set(GtkWidget *w, gchar *font_name);
void lj_run(LogJamDoc *doc);
void lj_do_loginout(LJWin *ljw);
void lj_update_usejournal(LJWin *ljw);
gboolean lj_confirm_lose_entry(LJWin *ljw);

void lj_clear_entry(LJWin *ljw);
void lj_open_entry(LJWin *ljw);
void lj_open_draft(LJWin *ljw);

gboolean lj_save_as_file(LJWin *ljw);
gboolean lj_save_as_draft(LJWin *ljw);
gboolean lj_save(LJWin *ljw);

void lj_load_entry(LJWin *ljw, Entry *entry);
void lj_submit_entry(LJWin *ljw);
void lj_save_entry_server(LJWin *ljw);

void lj_quit(LJWin *ljw);
void lj_autosave_init(LJWin *ljw);
void lj_autosave_stop(LJWin *ljw);

CFIndicator* make_cf_float(CFMgr *cfm, GtkWindow *parent);

#endif /* lj_h */


