/* logjam - a GTK client for LiveJournal.
 * Copyright (C) 2000-2003 Evan Martin <evan@livejournal.com>
 *
 * vim: tabstop=4 shiftwidth=4 noexpandtab :
 */

#include "config.h"

#include <gtk/gtk.h>

#if XMMS_TYPE == XMMS_TYPE_LINK
#include <xmmsctrl.h>
#elif XMMS_TYPE == XMMS_TYPE_RUNTIME
#include <gmodule.h>
#elif XMMS_TYPE == XMMS_TYPE_ERROR
#error XMMS_TYPE is set incorrectly by configure.in.  This is a bug!  Report it!
#endif

#include "conf.h"
#include "util.h"
#include "music.h"

GQuark
music_error_quark(void) {
	static GQuark quark = 0;
	if (quark == 0)
		quark = g_quark_from_static_string("music-error-quark");
	return quark;
}

#if XMMS_TYPE == XMMS_TYPE_RUNTIME
static GModule *xmms_gm = NULL; /* for g_module_open(). */

static gboolean
music_xmms_can_detect(GError **err) {
	if (!g_module_supported()) {
		g_set_error(err, MUSIC_ERROR, MUSIC_MODULE_ERROR,
				"Modules are not supported on this platform.");
		return FALSE;
	}

	if (xmms_gm == NULL) {
		const char *filename = "libxmms.so.1";
		xmms_gm = g_module_open(filename, G_MODULE_BIND_LAZY);
		if (!xmms_gm) {
			g_set_error(err, MUSIC_ERROR, MUSIC_OPEN_ERROR,
					"Error g_module_open()ing %s: %s",
					filename, g_module_error());
			return FALSE; /* no library; don't use xmms. */
		}
	}
	return TRUE;
}

static char*
music_xmms_detect(GError **err) {
	gint pos;
	gchar *text;
	char *utf8;
	/*
	 * we do some wackyness here to avoid statically linking with XMMS.
gint xmms_remote_get_playlist_pos(gint session);
gchar *xmms_remote_get_playlist_title(gint session, gint pos);
	*/
	static gint (*xrgpp)(gint) = NULL;
	static gchar* (*xrgpt)(gint, gint) = NULL;

	if (xrgpp == NULL || xrgpt == NULL) {
		const char *xrgppname = "xmms_remote_get_playlist_pos";
		const char *xrgptname = "xmms_remote_get_playlist_title";

		if (!g_module_symbol(xmms_gm,
					"xmms_remote_get_playlist_pos", (gpointer)&xrgpp) 
				|| xrgpp == NULL) {
			g_set_error(err, MUSIC_ERROR, MUSIC_SYMBOL_ERROR,
					"Error g_module_symbol()ing for %s: %s",
					xrgppname, g_module_error());
			return NULL;
		}
		if (!g_module_symbol(xmms_gm,
					"xmms_remote_get_playlist_title", (gpointer)&xrgpt)
				|| xrgpt == NULL) {
			g_set_error(err, MUSIC_ERROR, MUSIC_SYMBOL_ERROR,
					"Error g_module_symbol()ing for %s: %s",
					xrgptname, g_module_error());
			return NULL;
		}
	}
	
	pos = xrgpp(0);
	text = xrgpt(0, pos);

	if (!text) {
		g_set_error(err, MUSIC_ERROR, MUSIC_XMMS_ERROR,
				"XMMS xmms_remote_get_playlist_title() returned NULL.");
		return NULL;
	}

	utf8 = g_locale_to_utf8(text, -1, NULL, NULL, err);
	g_free(text);

	return utf8;
}
/* XMMS_TYPE_RUNTIME */
#elif XMMS_TYPE == XMMS_TYPE_LINK
static char*
music_xmms_detect(GError **err) {
	gint pos;
	gchar *text;
	char *utf8;

	pos = xmms_remote_get_playlist_pos(0);
	text = xmms_remote_get_playlist_title(0, pos);
	if (!text) {
		g_set_error(err, MUSIC_ERROR, MUSIC_XMMS_ERROR,
				"XMMS xmms_remote_get_playlist_title() returned NULL.");
		return NULL;
	}

	utf8 = g_locale_to_utf8(text, -1, NULL, NULL, err);
	g_free(text);

	return utf8;
}

static gboolean
music_xmms_can_detect(GError **err) {
	return TRUE;
}

#else

char*
music_xmms_detect(GError **err) {
	return NULL;
}
gboolean
music_xmms_can_detect(GError **err) {
	g_set_error(err, MUSIC_ERROR, MUSIC_CONFIGURE_ERROR,
			_("LogJam was not compiled with support for music."));
	return FALSE;
}

#endif /* XMMS_TYPE */ 

#ifndef G_OS_WIN32
static gboolean
music_command_can_detect(GError **err) {
	if (!conf.music_command || !conf.music_command[0]) {
		g_set_error(err, MUSIC_ERROR, MUSIC_COMMAND_LINE_ERROR,
				_("No music command line is set."));
		return FALSE;
	}
	return TRUE;
}

static char*
music_command_detect(GError **err) {
	gchar *std_out, *p;
	if (!g_spawn_command_line_sync(conf.music_command,
			&std_out, NULL, NULL, err))
		return NULL;
	/* we take the first line of this output. */
	for (p = std_out; *p; p++)
		if (*p == '\n') *p = 0;
	return std_out;
}

gboolean
music_can_detect(GError **err) {
	switch (conf.music_source) {
	case MUSIC_SOURCE_XMMS:
		return music_xmms_can_detect(err);
	case MUSIC_SOURCE_COMMAND:
		return music_command_can_detect(err);
	case MUSIC_SOURCE_NONE:
		return FALSE;
	}
	return FALSE; /* not reached. */
}

char*
music_detect(GError **err) {
	switch (conf.music_source) {
	case MUSIC_SOURCE_XMMS:
		return music_xmms_detect(err);
	case MUSIC_SOURCE_COMMAND:
		return music_command_detect(err);
	case MUSIC_SOURCE_NONE:
		return NULL;
	}
	return NULL; /* not reached. */
}
#else
gboolean music_can_detect(GError **err) {
	g_set_error(err, MUSIC_ERROR, MUSIC_UNIMPLEMENTED_ERROR,
			_("Music detection is not implemented on this platform."));
	return FALSE;
}
char* music_detect(GError **err) {
	g_set_error(err, MUSIC_ERROR, MUSIC_UNIMPLEMENTED_ERROR,
			_("Music detection is not implemented on this platform."));
	return FALSE;
}
#endif /* G_OS_WIN32 */

