/* logjam - a GTK client for LiveJournal.
 * Copyright (C) 2000-2003 Evan Martin <evan@livejournal.com>
 *
 * vim: tabstop=4 shiftwidth=4 noexpandtab :
 */

#ifndef network_h
#define network_h

#include <gtk/gtkwindow.h>
#include "protocol.h"

/* network -- medium-level interface to lj protocol.
 * takes a hash of request values, returns hash of lj
 * response, optionally with a "progress" dialog that
 * can be cancelled.
 */

typedef LJRequest NetRequest;
typedef LJResult  NetResult;

NetRequest* net_request_new(const char *mode);
NetResult* net_request_run_silent(NetRequest *request);
NetResult* net_request_run(NetRequest *request, const char *title, GtkWindow *parent);
NetResult* net_request_run_cli(NetRequest *request);
#define net_request_set(r, k, v) g_hash_table_insert(r, k, v)
#define net_request_seti(r, k, v) g_hash_table_insert(r, g_strdup(k), g_strdup_printf("%d", v))
#define net_request_sets(r, k, v) net_request_set(r, g_strdup(k), v)
#define net_request_copys(r, k, v) net_request_sets(r, k, g_strdup(v))
#define net_request_free(r) lj_protocol_request_free(r)

int net_result_succeeded(NetResult *result);
#define net_result_get(r, k) g_hash_table_lookup(r, k)
int net_result_geti(NetResult *result, const char *key);
char* net_result_getf(NetResult* result, const char *key, ...);
char* net_result_get_prefix(NetResult *result, const char *prefix, const char *key);
#define net_result_free(r) lj_protocol_result_free(r)

/* lower-level window-based network interface.
 * used by syncitems to have a custom progress dialog. */

#define NET_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_CAST((obj), net_window_get_type(), NetWindow))

typedef struct _NetWindow NetWindow;

GType      net_window_get_type(void);
GtkWidget* net_window_new(GtkWindow *parent, const char *title);
void       net_window_pack(NetWindow *nw, GtkWidget *contents);
NetResult* net_window_run_request(NetWindow *nw, NetRequest *request);
void       net_window_show_error(NetWindow *nw, const char *fmt, ...);

#endif /* network_h */
