/* logjam - a GTK client for LiveJournal.
 * Copyright (C) 2000-2003 Evan Martin <evan@livejournal.com>
 *
 * vim: tabstop=4 shiftwidth=4 noexpandtab :
 */

#ifndef util_h
#define util_h

#include <gtk/gtktable.h>
#include <gtk/gtktreemodel.h>
#include <gtk/gtkspinbutton.h>
#include <gtk/gtksizegroup.h>
#include <gtk/gtkwindow.h>
#include <gtk/gtkpaned.h>
#include <gtk/gtkdialog.h>

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "conf.h" /* for the "geometry" struct */

/* i18n stuff. */
#include "gettext.h"
#define _(String) gettext (String)
#define gettext_noop(String) String
#define N_(String) gettext_noop (String)

typedef enum {
	LJ_MSG_INFO,
	LJ_MSG_WARNING,
	LJ_MSG_ERROR
} MessageType;

/* converting to/from ljdate ("yyyy-mm-dd hh:mm:ss") and unix time */
char* tm_to_ljdate (struct tm *ptm);
char* tm_to_ljdate_noseconds(struct tm *ptm);
gboolean ljdate_to_tm(const char *ljtime, struct tm *ptm);

/* gtk helpers */
GtkWidget* lj_table_new(int rows, int cols);
GtkWidget* lj_table_label(GtkTable *table, int row, const char *text);
void       lj_table_content(GtkTable *table, int row, GtkWidget *content);
void       lj_table_label_content(GtkTable *table, int row, const char *text, GtkWidget *content);
void       lj_table_label_content_mne(GtkTable *table, int row, const char *text, GtkWidget *content, GtkWidget *mne);
void       lj_table_fillrow(GtkTable *table, int row, GtkWidget *content);
void       lj_spin_button_set(GtkSpinButton *w, gboolean numeric, gdouble range_low, gdouble range_high, gdouble increment_single, gdouble increment_page, gint digits);

void lj_win_set_size(GtkWindow *win, int width, int height);

GtkWidget* lj_filesel_new(const gchar *title, GtkWindow *parent);

void lj_dialog_set_contents_container(GtkDialog *dlg, GtkWidget *container);
GtkWidget* lj_dialog_set_contents(GtkDialog *dlg, GtkWidget *contents);

GtkWidget* lj_dialog_buttonbox_new(void);
void       lj_dialog_buttonbox_add(GtkWidget *box, GtkWidget *button);
GtkWidget* lj_dialog_buttonbox_button_with_label(GtkWidget *box, const char *label);
GtkWidget* lj_dialog_buttonbox_button_from_stock(GtkWidget *box, const char *id);
void       lj_dialog_set_contents_buttonbox(GtkWidget *dlg, GtkWidget *contents, GtkWidget *buttonbox);

int  lj_confirm(GtkWindow *parent, const char *title, const char *msg);
void lj_message(GtkWindow *parent, MessageType type, gboolean forgettable, const char *title, const char *message, ...);
void lj_messagebox(GtkWindow *parent, const char *title, const char *message);
void lj_warning(GtkWindow *parent, const char *msg, ...);

void lj_cli_note(const char *msg, ...);
/* field_strdup_printf("a %2 c %1", "d", "b") -> "a b c d" */
char* field_strdup_printf(char *fmt, char* fields[]);

void string_replace(char **dest, char *src);
void strtoupper(char *str);

/*
gdk_color_parse() does exactly this:
void hex_to_gdkcolor(const char *buf, GdkColor *c);
*/
void gdkcolor_to_hex(GdkColor *color, char* buf);

/* wrap this widget in a scrollarea */
GtkWidget* scroll_wrap(GtkWidget *w);

/* tie a geometry struct to a window */
void geometry_tie(GtkWidget *win, GeometryType g);
void geometry_tie_full(GeometryType g, GtkWindow *win, GtkPaned *paned);

unsigned char* urldecode(char *string);
char* urlencode(unsigned char *string);

#define unref_and_null(x) if (x) { g_object_unref(G_OBJECT(x)); x = NULL; }

gboolean verify_dir(const char *path, GError **err);
gboolean verify_path(char *path, gboolean include_last, GError **err);

void md5_hash(const char *src, char *dest);

gint text_sort_func(GtkTreeModel *model, GtkTreeIter  *a, GtkTreeIter  *b, gpointer data);

void lj_widget_set_visible(GtkWidget *w, gboolean visible);
void lj_widget_set_font(GtkWidget *w, const gchar *font_name);

gchar* gettext_translate_func(const gchar *path, gpointer data);

#define STACK(type, var) type actual_##var = {0}, *var = &actual_##var

/* produces an hbox that has a label and a widget, with optional
 * sizegroup, expansion setting, and mnemonic widget. */
GtkWidget* labelled_box_new_all(const char *caption, GtkWidget *w,
                                gboolean expand, GtkSizeGroup *sg,
                                GtkWidget *mne);
#define labelled_box_new_sg(c, w, sg) labelled_box_new_all(c, w, TRUE, sg, NULL)
#define labelled_box_new_expand(c, w, e) labelled_box_new_all(c, w, e, NULL, NULL)
#define labelled_box_new(c, w) labelled_box_new_all(c, w, TRUE, NULL, NULL)

/* make a short summary of a post, given subject and content. */
char* get_summary(char *subject, char *event);

/* like gtk_label_new, but appropriate for a form. */
GtkWidget* lj_form_label_new(char *text);

/* verify str is utf8, and if not replace bad chars with '.' */
void verify_utf8(char **str);

#endif /* util_h */
